@@ -1,9 +1,11 @@
 import asyncio
-from typing import List, Dict
+from typing import List, Dict, Optional
 from collections import deque, defaultdict
 from msglink import packets
 import hashlib
 import time
+from contextlib import suppress
+
 
 
 BUFFER_TIME_MS = 500
@@ -51,11 +53,11 @@ class Protocol(asyncio.Protocol):
         self._bytes_since_update = 0
         self._last_update_time = time.perf_counter()
         self._streams: List[Stream] = list()
-        self._transport: asyncio.WriteTransport = None
+        self._transport: Optional[asyncio.WriteTransport] = None
         self._input_buffer = bytearray()
         self._is_connected = False
         self._peer_roles = dict()
-        self._heartbeat_task: asyncio.Task = None
+        self._heartbeat_task: Optional[asyncio.Task] = None
         self._message_buffers: Dict[tuple, bytearray] = defaultdict(bytearray)
         self._post_buffers: Dict[tuple, bytearray] = defaultdict(bytearray)
         self._topics: Dict[bytes, str] = dict()
@@ -106,6 +108,13 @@ class Protocol(asyncio.Protocol):
     def close(self):
         self._transport.abort()
 
+    async def async_close(self):
+        unfinished = self._heartbeat_task
+        self.close()
+        if unfinished:
+            with suppress(asyncio.CancelledError):
+                await unfinished
+
     def is_connected(self):
         return self._is_connected
 
