@@ -23,6 +23,7 @@ class Router(msglink.Callbacks):
         self._running = False
         self._peer_discoveries: Set[Router.PeerDiscovery] = set()
         self._disconnect_waits: Set[Router.DisconnectWait] = set()
+        self._client_readers: Set[Router.ClientReader] = set()
         self._channel_readers: Set[Router.ChannelReader] = set()
         self._topic_readers: Set[Router.TopicReader] = set()
         self._established = False
@@ -94,6 +95,23 @@ class Router(msglink.Callbacks):
         await wait.future
         self._disconnect_waits.remove(wait)
 
+    async def read_peer_messages(self, peer: int):
+        if peer not in self._connected_peers:
+            return
+
+        cr = Router.ClientReader(peer)
+        self._client_readers.add(cr)
+
+        try:
+            while self._running:
+                msg = await cr.queue.get()
+                if msg is not None:
+                    yield msg
+                else:
+                    return
+        finally:
+            self._client_readers.remove(cr)
+
     async def read_peer_channel(self, peer: int, channel: int):
         if peer not in self._connected_peers:
             return
@@ -163,6 +181,11 @@ class Router(msglink.Callbacks):
             self.pid = peer
             self.future = asyncio.Future()
 
+    class ClientReader:
+        def __init__(self, peer: int):
+            self.pid = peer
+            self.queue = asyncio.Queue()
+
     class ChannelReader:
         def __init__(self, peer: int, channel: int):
             self.pid = peer
@@ -249,8 +272,12 @@ class Router(msglink.Callbacks):
                 cr.queue.put_nowait(None)
 
     def on_message(self, sender, role, channel, payload):
+        for cr in self._client_readers:
+            if cr.pid == sender:
+                cr.queue.put_nowait(payload)
+
         for cr in self._channel_readers:
-            if cr.pid == sender and cr.channel == cr.channel:
+            if cr.pid == sender and cr.channel == channel:
                 cr.queue.put_nowait(payload)
 
     def on_post(self, sender, role, topic, payload):
